import 'dart:io';

import 'package:yaml/yaml.dart';

import '../model/services_model.dart';
import '../core/setup.dart';

class Utils {
  static String coreUrl = "https://core.dative.me";
  static String installEndPoint = "install";

  static void changeAndroidPackageName(String newPackageName) async {
    final oldPackageName = await getApplicationIdFromBuildGradle();

    if (oldPackageName != null) {
      bool updated =
          await updatePackageAndMainActivity(oldPackageName, newPackageName);

      if (updated) {
        final gradleFile = File('android/app/build.gradle');
        String content = gradleFile.readAsStringSync();

        final RegExp pattern = RegExp(r'applicationId "[\w.]*"');
        content =
            content.replaceAll(pattern, 'applicationId "$newPackageName"');
        gradleFile.writeAsStringSync(content);

        final RegExp pattern1 = RegExp(r'namespace "[\w.]*"');
        content = content.replaceAll(pattern1, 'namespace "$newPackageName"');
        gradleFile.writeAsStringSync(content);
      } else {
        exit(1);
      }
    } else {
      print('ERROR: Could not read package name from Android gradle');
      exit(1);
    }
  }

  static Future<bool> updatePackageAndMainActivity(
      String oldPackageName, String newPackageName) async {
    String basePath = 'android/app/src/main/kotlin/';
    String oldPath = '$basePath${oldPackageName.replaceAll('.', '/')}';
    String newPath = '$basePath${newPackageName.replaceAll('.', '/')}';

    final oldDir = Directory(oldPath);
    final newDir = Directory(newPath);

    if (!oldDir.existsSync()) {
      print('Old package directory does not exist: $oldPath');
      return false;
    }

    try {
      // Create the new directory structure
      newDir.createSync(recursive: true);

      // Move each file from the old directory to the new one
      await for (var entity in oldDir.list(recursive: true)) {
        if (entity is File) {
          final relativePath = entity.path.substring(oldDir.path.length);
          final newPathFile = File('${newDir.path}$relativePath');
          await newPathFile.create(recursive: true);
          await entity.rename(newPathFile.path);
        }
      }

      // Delete the old directory and its potentially empty parent directories
      deleteDirectoryAndEmptyParents(oldDir);
    } catch (e) {
      print('Failed to move package directory: $e');
      return false;
    }

    // Update MainActivity.kt
    final mainActivityFile = File('${newDir.path}/MainActivity.kt');
    if (mainActivityFile.existsSync()) {
      String content = await mainActivityFile.readAsString();
      final newContent = content.replaceAll(
        RegExp(r'^package\s+[a-zA-Z0-9_.]+', multiLine: true),
        'package $newPackageName',
      );
      await mainActivityFile.writeAsString(newContent);
      print('Package(s) updated successfully.');
    } else {
      print('MainActivity file does not exist at ${mainActivityFile.path}');
      return false;
    }

    return true;
  }

  static void deleteDirectoryAndEmptyParents(Directory directory) {
    var currentDir = directory;
    while (currentDir.path != currentDir.parent.path) {
      // Stop if we reach the root
      if (currentDir.existsSync() && currentDir.listSync().isEmpty) {
        currentDir.deleteSync();
        currentDir = currentDir.parent;
      } else {
        break;
      }
    }
  }

  static void changeIosPackageName(String newPackageName) {
    final pbxprojFile = File('ios/Runner.xcodeproj/project.pbxproj');
    String content = pbxprojFile.readAsStringSync();
    final RegExp pattern = RegExp(r'PRODUCT_BUNDLE_IDENTIFIER = [\w.]*;');
    content = content.replaceAll(
        pattern, 'PRODUCT_BUNDLE_IDENTIFIER = $newPackageName;');
    pbxprojFile.writeAsStringSync(content);
  }

  static Future<String?> getApplicationIdFromBuildGradle() async {
    final file = File('android/app/build.gradle');
    if (!file.existsSync()) {
      print('build.gradle file does not exist');
      return null;
    }

    final lines = await file.readAsLines();
    String? applicationId;

    final appIdRegex = RegExp(r'applicationId "([\w.]+)"');

    for (final line in lines) {
      final match = appIdRegex.firstMatch(line);
      if (match != null && match.groupCount >= 1) {
        applicationId = match.group(1);
        break;
      }
    }

    return applicationId;
  }

  static String? getValidationKey() {
    final file = File('pubspec.yaml');
    final yamlString = file.readAsStringSync();
    final yaml = loadYaml(yamlString);

    return yaml['dative_core']?['config']?['validation_key'];
  }

  static String? getPurchaseCode() {
    final file = File('pubspec.yaml');
    final yamlString = file.readAsStringSync();
    final yaml = loadYaml(yamlString);

    return yaml['dative_core']?['config']?['purchase_code'];
  }

  //////// SETUP /////////

  static Future<bool> checkSetupInPubspec({String? packageNameiOS}) async {
    final file = File('pubspec.yaml');
    if (!await file.exists()) {
      print('pubspec.yaml file does not exist');
      return false;
    }

    final content = await file.readAsString();
    final yamlMap = loadYaml(content);

    if (yamlMap.containsKey('dative_core')) {
      final dativeCore = yamlMap['dative_core'];

      if (dativeCore.containsKey('setup') && dativeCore.containsKey('files')) {
        final setup = dativeCore['setup'];
        final files = dativeCore['files'];
        if (setup is YamlMap && setup.isNotEmpty && files is YamlMap && files.isNotEmpty) {

          ServicesModel googleServices =
          await Setup.extractGoogleServicesFile(files['google_file_ios']);

          final appName = setup['app_name'] ?? null;
          final facebookAppId = setup['facebook_app_id'] ?? null;

          await Setup.updateStringsXmlValues(
            appName: appName,
            facebookAppId: facebookAppId,
            facebookClientToken: setup['facebook_client_token'] ?? null,
            admobAppId: setup['admob_app_id_android'] ?? null,
          );

          await Setup.updateInfoPlist(
            appName: appName,
            facebookAppId: facebookAppId,
            facebookClientToken: setup['facebook_client_token'] ?? null,
            admobAppId: setup['admob_app_id_ios'] ?? null,
            packageName: packageNameiOS,
            googleClientId: googleServices.reversedClientId,
            googleAppId: googleServices.googleCodedAppId,
          );

          if(appName != null){
            await Setup.updateHtmlTitleAndAppTitle(appName: appName);
          }

          await Setup.updateAppConfigFile(
              appName: appName,
              pushGcm: googleServices.senderId,
              facebookAppId: facebookAppId,
              purchaseCode: getPurchaseCode()!,
              validationKey: getValidationKey()!,
          );

          return true;
        } else {
          print('Setup exists but has no values.');
          return false;
        }
      } else {
        print('Setup does not exist.');
        return false;
      }
    } else {
      print('dative_core does not exist in pubspec.yaml.');
      return false;
    }
  }

  static String? getGoogleServicesFileAndroid() {
    final file = File('pubspec.yaml');
    final yamlString = file.readAsStringSync();
    final yaml = loadYaml(yamlString);

    return yaml['dative_core']?['files']?['google_file_android'];
  }

  static String? getGoogleServicesFileIos() {
    final file = File('pubspec.yaml');
    final yamlString = file.readAsStringSync();
    final yaml = loadYaml(yamlString);

    return yaml['dative_core']?['files']?['google_file_ios'];
  }

  static Future<void> copyGoogleServicesFilesAndroid({String? source}) async {
    var sourceFilePath = source!;
    var destinationFilePath = 'android/app/google-services.json';
    var sourceFile = File(sourceFilePath);
    var newFile = await sourceFile.copy(destinationFilePath);

    print('Android Google Services file copied to ${newFile.path}');
  }

  static Future<void> copyGoogleServicesFilesIos({String? source}) async {
    var sourceFilePath = source!;
    var destinationFilePath = 'ios/GoogleService-Info.plist';
    var sourceFile = File(sourceFilePath);
    var newFile = await sourceFile.copy(destinationFilePath);

    print('iOS Google Services file copied to ${newFile.path}');
  }

  static String convertGoogleAppIdToCoded(String googleAppId) {
    final parts = googleAppId.split(':');
    final coded = 'app-' + parts.sublist(1).join('-');
    return coded;
  }
}
